/* The MIT License (Open Source Approved)

Copyright (c) 2010 R. A. Nagy

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.

 */
package benchmarkone;

import com.soft9000.logger.StatusLog;

/**
  * This code is a conversion to Java of the benchmark C# found at
 * http://sourceforge.net/projects/netlinawhetcpu, as originally
 * shared by paradoxon101.
 *
 * @author R.A. Nagy
 */
public class Operations {

    public static void ExeStandardLinpack(StatusLog log) {
        Linpack lin = new Linpack();
        lin.RunBenchmark();

        log.log("Mflop/s: " + lin.getMFlops());
        log.log("Sekunden: " + lin.getTime());
        log.log("Eps: " + lin.getEps());
        log.log("");
        log.log("Mflop/s ca.: " + lin.getMFlops() + ", per processor."); // Environment.ProcessorCount);
    }

    public static void ExeStandardWhetstone(StatusLog log) {
        Whetstone whet = new Whetstone();
        whet.setITERATIONS(100); // ITERATIONS/10 = Millions Whetstone instructions
        whet.setNumberOfCycles(100);

        int numberOfRuns = 10;
        float elapsedTime = 0;
        float meanTime = 0;
        float rating = 0;
        float meanRating = 0;
        int intRating = 0;
        for (int runNumber = 1; runNumber <= numberOfRuns; runNumber++) {
            // Call the Whetstone benchmark procedure
            // compute elapsed time
            elapsedTime = (float) (whet.StartCalc() / 1000);
            log.log("Run " + runNumber + ": Test (time for " + whet.getNumberOfCycles() + " cycles): " + (whet.getEndTime() - whet.getBeginTime()) + " millisec.");
            // sum time in milliseconds per cycle
            meanTime = meanTime + (elapsedTime * 1000 / whet.getNumberOfCycles());
            // Calculate the Whetstone rating based on the time for
            // the numbers of cycles just executed
            rating = (1000 * whet.getNumberOfCycles()) / elapsedTime;
            // Sum Whetstone rating
            meanRating = meanRating + rating;
            intRating = (int) rating;
            // Reset no_of_cycles for the next run using ten cycles more
            whet.incNumberOfCycles(10);
        }
        meanTime = meanTime / numberOfRuns;
        meanRating = meanRating / numberOfRuns;
        intRating = (int) meanRating;

        log.log("Number of Runs " + numberOfRuns);
        log.log("Average time per cycle " + meanTime + " millisec.");
        log.log("Average Whetstone Rating " + intRating + " KWIPS");
        log.log("Average Whetstone Rating " + (intRating / 1000) + " MWIPS");
    }
}
