/* The MIT License (Open Source Approved)

Copyright (c) 2010 R. A. Nagy

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.

*/
package benchmarkone;

/**
 * Signature as inspired by the C# Stopwatch mechanism.
 * 
 * @author R.A. Nagy
 */
public class Stopwatch {

    long lmsStart = 0L;
    long lmsStop = 0L;
    //
    // Summary:
    //     Gets the total elapsed time measured by the current instance, in milliseconds.
    //
    // Returns:
    //     A read-only long integer representing the total number of milliseconds measured
    //     by the current instance.

    public long ElapsedMilliseconds() {
        return (lmsStop - lmsStart);
    }
    //
    // Summary:
    //     Gets the total elapsed time measured by the current instance, in timer ticks.
    //
    // Returns:
    //     A read-only long integer representing the total number of timer ticks measured
    //     by the current instance.

    //public long ElapsedTicks() {
    //}
    //
    // Summary:
    //     Gets a value indicating whether the System.Diagnostics.Stopwatch timer is
    //     running.
    //
    // Returns:
    //     true if the System.Diagnostics.Stopwatch instance is currently running and
    //     measuring elapsed time for an interval; otherwise, false.
    public boolean IsRunning() {
        if (lmsStop != 0L && lmsStop == 0L) {
            return true;
        }
        return false;
    }

    // Summary:
    //     Gets the current number of ticks in the timer mechanism.
    //
    // Returns:
    //     A long integer representing the tick counter value of the underlying timer
    //     mechanism.
    public static long GetTimestamp() {
        return System.currentTimeMillis();
    }
    //
    // Summary:
    //     Stops time interval measurement and resets the elapsed time to zero.

    public void Reset() {
        this.lmsStart = this.lmsStop = 0L;
    }
    //
    // Summary:
    //     Starts, or resumes, measuring elapsed time for an interval.

    public void Start() {
        this.lmsStart = System.currentTimeMillis();
    }
    //
    // Summary:
    //     Stops measuring elapsed time for an interval.

    public void Stop() {
        this.lmsStop = System.currentTimeMillis();
    }
    //
    // Summary:
    //     Initializes a new System.Diagnostics.Stopwatch instance, sets the elapsed
    //     time property to zero, and starts measuring elapsed time.
    //
    // Returns:
    //     A System.Diagnostics.Stopwatch that has just begun measuring elapsed time.

    public static Stopwatch StartNew() {
        Stopwatch sw = new Stopwatch();
        sw.Start();
        return sw;
    }
}
